@@ -1,4 +1,4 @@
-#!/bin/sh -e
+#!/bin/bash
 #
 # This script builds all library dependencies of OpenSCAD for Mac OS X.
 # The libraries will be build in 64-bit (and optionally 32-bit mode) mode
@@ -6,18 +6,18 @@
 # 
 # This script must be run from the OpenSCAD source root directory
 #
-# Usage: macosx-build-dependencies.sh [-6lcd]
+# Usage: macosx-build-dependencies.sh [-6lcd] [<package>]
 #  -6   Build only 64-bit binaries
 #  -l   Force use of LLVM compiler
 #  -c   Force use of clang compiler
 #  -d   Build for deployment (if not specified, e.g. Sparkle won't be built)
+#  -f   Force build even if package is installed
 #
 # Prerequisites:
 # - MacPorts: curl, cmake
 #
 # FIXME:
 # o Verbose option
-# o Force rebuild vs. only rebuild changes
 #
 
 BASEDIR=$PWD/../libraries
@@ -30,22 +30,145 @@ OPTION_LLVM=false
 OPTION_CLANG=false
 OPTION_GCC=false
 OPTION_DEPLOY=false
-export QMAKESPEC=macx-g++
+OPTION_FORCE=0
+
+PACKAGES=(
+    "eigen 3.2.4"
+    "gmp 5.1.3"
+    "mpfr 3.1.2"
+    "boost 1.57.0"
+    "qt5 5.4.1"
+    "qscintilla 2.8.4"
+    # NB! For eigen, also update the path in the function
+    # NB! For CGAL, also update the actual download URL in the function
+    "cgal 4.5.2"
+    "glew 1.12.0"
+    "gettext 0.19.4"
+    "libffi 3.2.1"
+    "glib2 2.42.1"
+    "opencsg 1.4.0"
+    "freetype 2.5.5"
+    "ragel 6.9"
+    "harfbuzz 0.9.37"
+    "libxml2 2.9.2"
+    "fontconfig 2.11.1"
+)
+DEPLOY_PACKAGES=(
+    "sparkle Cocoanetics:1e7dcb1a48b96d1a8c62100b5864bd50211cbae1"
+)
 
 printUsage()
 {
-  echo "Usage: $0 [-6lcd]"
+  echo "Usage: $0 [-6lcd] [<package>]"
   echo
   echo "  -6   Build only 64-bit binaries"
   echo "  -l   Force use of LLVM compiler"
   echo "  -c   Force use of clang compiler"
   echo "  -d   Build for deployment"
+  echo "  -f   Force build even if package is installed"
+  echo
+  echo "  If <package> is not specified, builds all packages"
+}
+
+# Outputs all package names
+all_packages()
+{
+    for i in $(seq 0 $(( ${#PACKAGES[@]} - 1 )) ); do
+        local p=${PACKAGES[$i]}
+        echo -n "${p%%\ *} " # Cut at first space
+    done
+}
+
+# Usage: package_version <package>
+# Outputs the package version for the given package
+package_version()
+{
+    for i in $(seq 0 $(( ${#PACKAGES[@]} - 1 )) ); do
+        local p=${PACKAGES[$i]}
+        if [ "$1" = "${p%%\ *}" ]; then
+            echo "${p#*\ }" # cut until first space
+            return 0
+        fi
+    done
+    return 1
+}
+
+# Usage: build <package> <version>
+build()
+{
+    local package=$1
+    local version=$2
+
+    local should_install=$(( $OPTION_FORCE == 1 ))
+    if [[ $should_install == 0 ]]; then
+        is_installed $package $version
+        should_install=$?
+    fi
+    if [[ $should_install == 1 ]]; then
+        set -e
+        build_$package $version
+        set +e
+    fi
+    
+}
+
+# Usage: is_installed <package> [<version>]
+# Returns success (0) if the/a version of the package is already installed
+is_installed()
+{
+    if check_$1 $2; then
+      echo "$1 already installed - not building"
+      return 0
+    fi
+    return 1
+}
+
+# Usage: check_dir <dir>
+# Checks if $DEPLOYDIR/<dir> exists and is a folder
+# Returns success (0) if the folder exists
+check_dir()
+{
+    test -d "$DEPLOYDIR/$1"
+}
+
+# Usage: check_file <file>
+# Checks if $DEPLOYDIR/<file> exists and is a file
+# Returns success (0) if the file exists
+check_file()
+{
+    test -f "$DEPLOYDIR/$1"
+}
+
+
+patch_qt_disable_core_wlan()
+{
+  version="$1"
+
+  patch -p1 <<END-OF-PATCH
+--- qt-everywhere-opensource-src-4.8.5/src/plugins/bearer/bearer.pro.orig	2013-11-01 19:04:29.000000000 +0100
++++ qt-everywhere-opensource-src-4.8.5/src/plugins/bearer/bearer.pro	2013-10-31 21:53:00.000000000 +0100
+@@ -12,7 +12,7 @@
+ #win32:SUBDIRS += nla
+ win32:SUBDIRS += generic
+ win32:!wince*:SUBDIRS += nativewifi
+-macx:contains(QT_CONFIG, corewlan):SUBDIRS += corewlan
++#macx:contains(QT_CONFIG, corewlan):SUBDIRS += corewlan
+ macx:SUBDIRS += generic
+ symbian:SUBDIRS += symbian
+ blackberry:SUBDIRS += blackberry
+END-OF-PATCH
 }
 
 # FIXME: Support gcc/llvm/clang flags. Use -platform <whatever> to make this work? kintel 20130117
 build_qt()
 {
   version=$1
+
+  if [ -d $DEPLOYDIR/lib/QtCore.framework ]; then
+    echo "qt already installed. not building"
+    return
+  fi
+
   echo "Building Qt" $version "..."
   cd $BASEDIR/src
   rm -rf qt-everywhere-opensource-src-$version
@@ -65,8 +188,74 @@ build_qt()
   if $OPTION_32BIT; then
     QT_32BIT="-arch x86"
   fi
-  ./configure -prefix $DEPLOYDIR -release $QT_32BIT -arch x86_64 -opensource -confirm-license $PLATFORM -fast -no-qt3support -no-svg -no-phonon -no-audio-backend -no-multimedia -no-javascript-jit -no-script -no-scripttools -no-declarative -no-xmlpatterns -nomake demos -nomake examples -nomake docs -nomake translations -no-webkit
+  case "$OSX_VERSION" in
+    9)
+      # libtiff fails in the linker step with Mavericks / XCode 5.0.1
+      MACOSX_RELEASE_OPTIONS=-no-libtiff
+      # wlan support bails out with lots of compiler errors, disable it for the build
+      patch_qt_disable_core_wlan "$version"
+      ;;
+    *)
+      MACOSX_RELEASE_OPTIONS=
+      ;;
+  esac
+  ./configure -prefix $DEPLOYDIR -release $QT_32BIT -arch x86_64 -opensource -confirm-license $PLATFORM -fast -no-qt3support -no-svg -no-phonon -no-audio-backend -no-multimedia -no-javascript-jit -no-script -no-scripttools -no-declarative -no-xmlpatterns -nomake demos -nomake examples -nomake docs -nomake translations -no-webkit $MACOSX_RELEASE_OPTIONS
+  make -j"$NUMCPU" install
+}
+
+check_qt5()
+{
+    check_dir lib/QtCore.framework
+}
+
+build_qt5()
+{
+  version=$1
+
+  echo "Building Qt" $version "..."
+  cd $BASEDIR/src
+  v=(${version//./ }) # Split into array
+  rm -rf qt-everywhere-opensource-src-$version
+  if [ ! -f qt-everywhere-opensource-src-$version.tar.gz ]; then
+     curl -O -L http://download.qt-project.org/official_releases/qt/${v[0]}.${v[1]}/$version/single/qt-everywhere-opensource-src-$version.tar.gz
+  fi
+  tar xzf qt-everywhere-opensource-src-$version.tar.gz
+  cd qt-everywhere-opensource-src-$version
+  ./configure -prefix $DEPLOYDIR -release -opensource -confirm-license \
+		-nomake examples -nomake tests \
+		-no-xcb -no-c++11 -no-glib -no-harfbuzz -no-sql-db2 -no-sql-ibase -no-sql-mysql -no-sql-oci -no-sql-odbc \
+		-no-sql-psql -no-sql-sqlite2 -no-sql-tds -no-cups -no-qml-debug \
+		-skip activeqt -skip connectivity -skip declarative -skip doc \
+		-skip enginio -skip graphicaleffects -skip location -skip multimedia \
+		-skip quick1 -skip quickcontrols -skip script -skip sensors -skip serialport \
+		-skip svg -skip webkit -skip webkit-examples -skip websockets -skip xmlpatterns
+  make -j"$NUMCPU" install
+}
+
+check_qscintilla()
+{
+    check_file include/Qsci/qsciscintilla.h 
+}
+
+build_qscintilla()
+{
+  version=$1
+  echo "Building QScintilla" $version "..."
+  cd $BASEDIR/src
+  rm -rf QScintilla-gpl-$version
+  if [ ! -f QScintilla-gpl-$version.tar.gz ]; then
+    curl -LO http://downloads.sourceforge.net/project/pyqt/QScintilla2/QScintilla-$version/QScintilla-gpl-$version.tar.gz
+  fi
+  tar xzf QScintilla-gpl-$version.tar.gz
+  cd QScintilla-gpl-$version/Qt4Qt5
+  qmake qscintilla.pro
   make -j6 install
+  install_name_tool -id $DEPLOYDIR/lib/libqscintilla2.dylib $DEPLOYDIR/lib/libqscintilla2.dylib
+}
+
+check_gmp()
+{
+    check_file lib/libgmp.dylib
 }
 
 # Hack warning: gmplib is built separately in 32-bit and 64-bit mode
@@ -75,6 +264,7 @@ build_qt()
 build_gmp()
 {
   version=$1
+
   echo "Building gmp" $version "..."
   cd $BASEDIR/src
   rm -rf gmp-$version
@@ -166,11 +356,17 @@ EOF
   cp x86_64/include/gmpxx.h include/
 }
 
+check_mpfr()
+{
+    check_file include/mpfr.h
+}
+
 # As with gmplib, mpfr is built separately in 32-bit and 64-bit mode and then merged
 # afterwards.
 build_mpfr()
 {
   version=$1
+
   echo "Building mpfr" $version "..."
   cd $BASEDIR/src
   rm -rf mpfr-$version
@@ -208,9 +404,15 @@ build_mpfr()
   cp x86_64/include/mpf2mpfr.h include/
 }
 
+check_boost()
+{
+    check_file lib/libboost_system.dylib
+}
+
 build_boost()
 {
   version=$1
+
   bversion=`echo $version | tr "." "_"`
   echo "Building boost" $version "..."
   cd $BASEDIR/src
@@ -227,12 +429,12 @@ build_boost()
   fi
   if $USING_LLVM; then
     BOOST_TOOLSET="toolset=darwin-llvm"
-    echo "using darwin : llvm : llvm-g++ ;" >> tools/build/v2/user-config.jam 
+    echo "using darwin : llvm : llvm-g++ ;" >> tools/build/user-config.jam 
   elif $USING_CLANG; then
     BOOST_TOOLSET="toolset=clang"
-    echo "using clang ;" >> tools/build/v2/user-config.jam 
+    echo "using clang ;" >> tools/build/user-config.jam 
   fi
-  ./b2 -j6 -d+2 $BOOST_TOOLSET cflags="-mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64 $BOOST_EXTRA_FLAGS" linkflags="-mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64 $BOOST_EXTRA_FLAGS -headerpad_max_install_names" install
+  ./b2 -j"$NUMCPU" -d+2 $BOOST_TOOLSET cflags="-mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64 $BOOST_EXTRA_FLAGS" linkflags="-mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64 $BOOST_EXTRA_FLAGS -headerpad_max_install_names" install
   install_name_tool -id $DEPLOYDIR/lib/libboost_thread.dylib $DEPLOYDIR/lib/libboost_thread.dylib 
   install_name_tool -change libboost_system.dylib $DEPLOYDIR/lib/libboost_system.dylib $DEPLOYDIR/lib/libboost_thread.dylib 
   install_name_tool -change libboost_chrono.dylib $DEPLOYDIR/lib/libboost_chrono.dylib $DEPLOYDIR/lib/libboost_thread.dylib 
@@ -245,15 +447,25 @@ build_boost()
 
 }
 
+check_cgal()
+{
+    check_file lib/libCGAL.dylib
+}
+
 build_cgal()
 {
   version=$1
+
   echo "Building CGAL" $version "..."
   cd $BASEDIR/src
   rm -rf CGAL-$version
   if [ ! -f CGAL-$version.tar.gz ]; then
-    # 4.3
-    curl -O https://gforge.inria.fr/frs/download.php/32994/CGAL-$version.tar.gz
+    # 4.5.2
+    curl -O https://gforge.inria.fr/frs/download.php/file/34512/CGAL-$version.tar.gz
+    # 4.5.1 curl -O https://gforge.inria.fr/frs/download.php/file/34400/CGAL-$version.tar.gz
+    # 4.5 curl -O https://gforge.inria.fr/frs/download.php/file/34149/CGAL-$version.tar.gz
+    # 4.4 curl -O https://gforge.inria.fr/frs/download.php/file/33525/CGAL-$version.tar.gz
+    # 4.3 curl -O https://gforge.inria.fr/frs/download.php/32994/CGAL-$version.tar.gz
     # 4.2 curl -O https://gforge.inria.fr/frs/download.php/32359/CGAL-$version.tar.gz
     # 4.1 curl -O https://gforge.inria.fr/frs/download.php/31641/CGAL-$version.tar.gz
     # 4.1-beta1 curl -O https://gforge.inria.fr/frs/download.php/31348/CGAL-$version.tar.gz
@@ -269,16 +481,22 @@ build_cgal()
     CGAL_EXTRA_FLAGS=";i386"
   fi
   cmake -DCMAKE_INSTALL_PREFIX=$DEPLOYDIR -DGMP_INCLUDE_DIR=$DEPLOYDIR/include -DGMP_LIBRARIES=$DEPLOYDIR/lib/libgmp.dylib -DGMPXX_LIBRARIES=$DEPLOYDIR/lib/libgmpxx.dylib -DGMPXX_INCLUDE_DIR=$DEPLOYDIR/include -DMPFR_INCLUDE_DIR=$DEPLOYDIR/include -DMPFR_LIBRARIES=$DEPLOYDIR/lib/libmpfr.dylib -DWITH_CGAL_Qt3=OFF -DWITH_CGAL_Qt4=OFF -DWITH_CGAL_ImageIO=OFF -DBUILD_SHARED_LIBS=TRUE -DCMAKE_OSX_DEPLOYMENT_TARGET="$MAC_OSX_VERSION_MIN" -DCMAKE_OSX_ARCHITECTURES="x86_64$CGAL_EXTRA_FLAGS" -DBOOST_ROOT=$DEPLOYDIR -DBoost_USE_MULTITHREADED=false
-  make -j4
+  make -j"$NUMCPU" install
   make install
   install_name_tool -id $DEPLOYDIR/lib/libCGAL.dylib $DEPLOYDIR/lib/libCGAL.dylib
   install_name_tool -id $DEPLOYDIR/lib/libCGAL_Core.dylib $DEPLOYDIR/lib/libCGAL_Core.dylib
   install_name_tool -change $PWD/lib/libCGAL.9.dylib $DEPLOYDIR/lib/libCGAL.dylib $DEPLOYDIR/lib/libCGAL_Core.dylib
 }
 
+check_glew()
+{
+    check_file lib/libGLEW.dylib
+}
+
 build_glew()
 {
   version=$1
+
   echo "Building GLEW" $version "..."
   cd $BASEDIR/src
   rm -rf glew-$version
@@ -294,70 +512,15 @@ build_glew()
   make GLEW_DEST=$DEPLOYDIR CC=$CC CFLAGS.EXTRA="-no-cpp-precomp -dynamic -fno-common -mmacosx-version-min=$MAC_OSX_VERSION_MIN $GLEW_EXTRA_FLAGS -arch x86_64" LDFLAGS.EXTRA="-mmacosx-version-min=$MAC_OSX_VERSION_MIN $GLEW_EXTRA_FLAGS -arch x86_64" STRIP= install
 }
 
-build_libffi()
+check_opencsg()
 {
-  version="$1"
-
-  if [ -e "$DEPLOYDIR/lib/libffi.a" ]; then
-    echo "libffi already installed. not building"
-    return
-  fi
-
-  echo "Building libffi $version..."
-  cd "$BASEDIR"/src
-  rm -rf "libffi-$version"
-  if [ ! -f "libffi-$version.tar.gz" ]; then
-    curl --insecure -LO "ftp://sourceware.org/pub/libffi/libffi-$version.tar.gz"
-  fi
-  tar xzf "libffi-$version.tar.gz"
-  cd "libffi-$version"
-  ./configure --prefix="$DEPLOYDIR"
-  make -j4
-  make install
-}
-
-build_gettext()
-{
-  version=$1
-  echo "Building gettext $version..."
-
-  cd "$BASEDIR"/src
-  rm -rf "gettext-$version"
-  if [ ! -f "glib-$version.tar.xz" ]; then
-    curl --insecure -LO "http://ftpmirror.gnu.org/gettext/gettext-$version.tar.gz"
-  fi
-  tar xzf "gettext-$version.tar.gz"
-  cd "gettext-$version"
-
-  ./configure --prefix="$DEPLOYDIR"
-  make -j4
-  make install
-}
-
-build_glib2()
-{
-  version=$1
-  echo "Building glib2 $version..."
-
-  cd "$BASEDIR"/src
-  rm -rf "glib-$version"
-  maj_min_version="${version%.*}" #Drop micro
-  if [ ! -f "glib-$version.tar.xz" ]; then
-    curl --insecure -LO "http://ftp.gnome.org/pub/gnome/sources/glib/$maj_min_version/glib-$version.tar.xz"
-  fi
-  tar xJf "glib-$version.tar.xz"
-  cd "glib-$version"
-
-  export PKG_CONFIG_LIBDIR="$DEPLOYDIR/lib/pkgconfig"
-  ./configure --disable-gtk-doc --disable-man --prefix="$DEPLOYDIR" CFLAGS="-I$DEPLOYDIR/include" LDFLAGS="-L$DEPLOYDIR/lib"
-  unset PKG_CONFIG_LIBDIR
-  make -j4
-  make install
+    check_file lib/libopencsg.dylib
 }
 
 build_opencsg()
 {
   version=$1
+
   echo "Building OpenCSG" $version "..."
   cd $BASEDIR/src
   rm -rf OpenCSG-$version
@@ -370,13 +533,27 @@ build_opencsg()
   if $OPTION_32BIT; then
     OPENCSG_EXTRA_FLAGS="x86"
   fi
-  OPENSCAD_LIBRARIES=$DEPLOYDIR qmake -r CONFIG+="x86_64 $OPENCSG_EXTRA_FLAGS"
+  qmake -r QMAKE_CXXFLAGS+="-I$DEPLOYDIR/include" QMAKE_LFLAGS+="-L$DEPLOYDIR/lib" CONFIG+="x86_64 $OPENCSG_EXTRA_FLAGS" DESTDIR=$DEPLOYDIR
   make install
 }
 
+# Usage: func [<version>]
+check_eigen()
+{
+    # To check version:
+    # include/eigen3/Eigen/src/Core/util/Macros.h:
+    #  #define EIGEN_WORLD_VERSION 3
+    #  #define EIGEN_MAJOR_VERSION 2
+    #  #define EIGEN_MINOR_VERSION 3
+
+    check_dir include/eigen3
+}
+
+# Usage: func <version>
 build_eigen()
 {
   version=$1
+
   echo "Building eigen" $version "..."
   cd $BASEDIR/src
   rm -rf eigen-$version
@@ -385,7 +562,12 @@ build_eigen()
   if [ $version = "3.1.2" ]; then EIGENDIR=eigen-eigen-5097c01bcdc4;
   elif [ $version = "3.1.3" ]; then EIGENDIR=eigen-eigen-2249f9c22fe8;
   elif [ $version = "3.1.4" ]; then EIGENDIR=eigen-eigen-36bf2ceaf8f5;
-  elif [ $version = "3.2.0" ]; then EIGENDIR=eigen-eigen-ffa86ffb5570; fi
+  elif [ $version = "3.2.0" ]; then EIGENDIR=eigen-eigen-ffa86ffb5570;
+  elif [ $version = "3.2.1" ]; then EIGENDIR=eigen-eigen-6b38706d90a9;
+  elif [ $version = "3.2.2" ]; then EIGENDIR=eigen-eigen-1306d75b4a21;
+  elif [ $version = "3.2.3" ]; then EIGENDIR=eigen-eigen-36fd1ba04c12;
+  elif [ $version = "3.2.4" ]; then EIGENDIR=eigen-eigen-10219c95fe65;
+  fi
 
   if [ $EIGENDIR = "none" ]; then
     echo Unknown eigen version. Please edit script.
@@ -405,19 +587,28 @@ build_eigen()
   if $OPTION_32BIT; then
     EIGEN_EXTRA_FLAGS=";i386"
   fi
-  cmake -DCMAKE_INSTALL_PREFIX=$DEPLOYDIR -DEIGEN_BUILD_LIB=ON -DBUILD_SHARED_LIBS=FALSE -DCMAKE_OSX_DEPLOYMENT_TARGET="$MAC_OSX_VERSION_MIN" -DCMAKE_OSX_ARCHITECTURES="x86_64$EIGEN_EXTRA_FLAGS" ..
-  make -j4
-  make install
+  cmake -DCMAKE_INSTALL_PREFIX=$DEPLOYDIR -DEIGEN_TEST_NOQT=TRUE -DCMAKE_OSX_DEPLOYMENT_TARGET="$MAC_OSX_VERSION_MIN" -DCMAKE_OSX_ARCHITECTURES="x86_64$EIGEN_EXTRA_FLAGS" ..
+  make -j"$NUMCPU" install
 }
 
+check_sparkle()
+{
+    check_file lib/Sparkle.framework/Sparkle
+}
+
+# Usage: build_sparkle <githubuser>:<commitID>
 build_sparkle()
 {
+  v=$1
+  github=${1%%:*}  # Cut at first colon
+  version=${1#*:}  # cut until first colon
+
+  echo "Building Sparkle" $version "..."
+
   # Let Sparkle use the default compiler
   unset CC
   unset CXX
-  github=$1
-  version=$2
-  echo "Building Sparkle" $version "..."
+
   cd $BASEDIR/src
   rm -rf Sparkle-$version
   if [ ! -f Sparkle-$version.zip ]; then
@@ -436,24 +627,226 @@ build_sparkle()
   install_name_tool -id $DEPLOYDIR/lib/Sparkle.framework/Versions/A/Sparkle $DEPLOYDIR/lib/Sparkle.framework/Sparkle
 }
 
+check_freetype()
+{
+    check_file lib/libfreetype.dylib
+}
+
+build_freetype()
+{
+  version="$1"
+  extra_config_flags="--without-png"
+
+  echo "Building freetype $version..."
+  cd "$BASEDIR"/src
+  rm -rf "freetype-$version"
+  if [ ! -f "freetype-$version.tar.gz" ]; then
+    curl --insecure -LO "http://downloads.sourceforge.net/project/freetype/freetype2/$version/freetype-$version.tar.gz"
+  fi
+  tar xzf "freetype-$version.tar.gz"
+  cd "freetype-$version"
+
+  export FREETYPE_CFLAGS="-I$DEPLOYDIR/include -I$DEPLOYDIR/include/freetype2"
+  export FREETYPE_LIBS="-L$DEPLOYDIR/lib -lfreetype"
+  PKG_CONFIG_LIBDIR="$DEPLOYDOR/lib/pkgconfig" ./configure --prefix="$DEPLOYDIR" CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN $extra_config_flags
+  make -j"$NUMCPU"
+  make install
+}
+ 
+check_libxml2()
+{
+    check_file lib/libxml2.dylib
+}
+
+build_libxml2()
+{
+  version="$1"
+
+  echo "Building libxml2 $version..."
+  cd "$BASEDIR"/src
+  rm -rf "libxml2-$version"
+  if [ ! -f "libxml2-$version.tar.gz" ]; then
+    curl --insecure -LO "ftp://xmlsoft.org/libxml2/libxml2-$version.tar.gz"
+  fi
+  tar xzf "libxml2-$version.tar.gz"
+  cd "libxml2-$version"
+  ./configure --prefix="$DEPLOYDIR" --with-zlib=/usr -without-lzma --without-ftp --without-http --without-python CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN
+  make -j$NUMCPU
+  make install
+}
+
+check_fontconfig()
+{
+    check_file lib/libfontconfig.dylib
+}
+
+build_fontconfig()
+{
+  version=$1
+
+  echo "Building fontconfig $version..."
+  cd "$BASEDIR"/src
+  rm -rf "fontconfig-$version"
+  if [ ! -f "fontconfig-$version.tar.gz" ]; then
+    curl --insecure -LO "http://www.freedesktop.org/software/fontconfig/release/fontconfig-$version.tar.gz"
+  fi
+  tar xzf "fontconfig-$version.tar.gz"
+  cd "fontconfig-$version"
+  export PKG_CONFIG_PATH="$DEPLOYDIR/lib/pkgconfig"
+  ./configure --prefix="$DEPLOYDIR" --enable-libxml2 CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN
+  unset PKG_CONFIG_PATH
+  make -j$NUMCPU
+  make install
+}
+
+check_libffi()
+{
+    check_file lib/libffi.dylib
+}
+
+build_libffi()
+{
+  version="$1"
+
+  echo "Building libffi $version..."
+  cd "$BASEDIR"/src
+  rm -rf "libffi-$version"
+  if [ ! -f "libffi-$version.tar.gz" ]; then
+    curl --insecure -LO "ftp://sourceware.org/pub/libffi/libffi-$version.tar.gz"
+  fi
+  tar xzf "libffi-$version.tar.gz"
+  cd "libffi-$version"
+  ./configure --prefix="$DEPLOYDIR"
+  make -j$NUMCPU
+  make install
+}
+
+check_gettext()
+{
+    check_file lib/libgettextlib.dylib
+}
+
+build_gettext()
+{
+  version="$1"
+
+  echo "Building gettext $version..."
+  cd "$BASEDIR"/src
+  rm -rf "gettext-$version"
+  if [ ! -f "gettext-$version.tar.xz" ]; then
+    curl --insecure -LO "http://ftpmirror.gnu.org/gettext/gettext-$version.tar.gz"
+  fi
+  tar xzf "gettext-$version.tar.gz"
+  cd "gettext-$version"
+
+  ./configure --prefix="$DEPLOYDIR" CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN
+  make -j$NUMCPU
+  make install
+}
+
+check_glib2()
+{
+    check_file lib/libglib-2.0.dylib
+}
+
+build_glib2()
+{
+  version="$1"
+
+  echo "Building glib2 $version..."
+
+  cd "$BASEDIR"/src
+  rm -rf "glib-$version"
+  maj_min_version="${version%.*}" #Drop micro
+  if [ ! -f "glib-$version.tar.xz" ]; then
+    curl --insecure -LO "http://ftp.gnome.org/pub/gnome/sources/glib/$maj_min_version/glib-$version.tar.xz"
+  fi
+  tar xJf "glib-$version.tar.xz"
+  cd "glib-$version"
+
+  export PKG_CONFIG_PATH="$DEPLOYDIR/lib/pkgconfig"
+  ./configure --disable-gtk-doc --disable-man --prefix="$DEPLOYDIR" CFLAGS="-I$DEPLOYDIR/include -mmacosx-version-min=$MAC_OSX_VERSION_MIN" LDFLAGS="-L$DEPLOYDIR/lib -mmacosx-version-min=$MAC_OSX_VERSION_MIN"
+  unset PKG_CONFIG_PATH
+  make -j$NUMCPU
+  make install
+}
+
+check_ragel()
+{
+    check_file bin/ragel
+}
+
+build_ragel()
+{
+  version=$1
+
+  echo "Building ragel $version..."
+  cd "$BASEDIR"/src
+  rm -rf "ragel-$version"
+  if [ ! -f "ragel-$version.tar.gz" ]; then
+    curl --insecure -LO "http://www.colm.net/files/ragel/ragel-$version.tar.gz"
+  fi
+  tar xzf "ragel-$version.tar.gz"
+  cd "ragel-$version"
+  sed -e "s/setiosflags(ios::right)/std::&/g" ragel/javacodegen.cpp > ragel/javacodegen.cpp.new && mv ragel/javacodegen.cpp.new ragel/javacodegen.cpp
+  ./configure --prefix="$DEPLOYDIR"
+  make -j$NUMCPU
+  make install
+}
+
+check_harfbuzz()
+{
+    check_file lib/libharfbuzz.dylib
+}
+
+build_harfbuzz()
+{
+  version=$1
+  extra_config_flags="--with-coretext=auto --with-glib=no"
+
+  echo "Building harfbuzz $version..."
+  cd "$BASEDIR"/src
+  rm -rf "harfbuzz-$version"
+  if [ ! -f "harfbuzz-$version.tar.gz" ]; then
+    curl --insecure -LO "http://cgit.freedesktop.org/harfbuzz/snapshot/harfbuzz-$version.tar.gz"
+  fi
+  tar xzf "harfbuzz-$version.tar.gz"
+  cd "harfbuzz-$version"
+  # disable doc directories as they make problems on Mac OS Build
+  sed -e "s/SUBDIRS = src util test docs/SUBDIRS = src util test/g" Makefile.am > Makefile.am.bak && mv Makefile.am.bak Makefile.am
+  sed -e "s/^docs.*$//" configure.ac > configure.ac.bak && mv configure.ac.bak configure.ac
+  PKG_CONFIG_LIBDIR="$DEPLOYDIR/lib/pkgconfig" ./autogen.sh --prefix="$DEPLOYDIR" --with-freetype=yes --with-gobject=no --with-cairo=no --with-icu=no CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN CXXFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN $extra_config_flags
+  make -j$NUMCPU
+  make install
+}
+
 if [ ! -f $OPENSCADDIR/openscad.pro ]; then
   echo "Must be run from the OpenSCAD source root directory"
   exit 0
 fi
+OPENSCAD_SCRIPTDIR=$PWD/scripts
 
-while getopts '6lcd' c
+while getopts '6lcdf' c
 do
   case $c in
     6) OPTION_32BIT=false;;
     l) OPTION_LLVM=true;;
     c) OPTION_CLANG=true;;
     d) OPTION_DEPLOY=true;;
+    f) OPTION_FORCE=1;;
+    *) printUsage;exit 1;;
   esac
 done
 
+OPTION_PACKAGES="${@:$OPTIND}"
+
 OSX_VERSION=`sw_vers -productVersion | cut -d. -f2`
-if (( $OSX_VERSION >= 8 )); then
-  echo "Detected Mountain Lion (10.8) or later"
+if (( $OSX_VERSION >= 10 )); then
+  echo "Detected Yosemite (10.10) or later"
+elif (( $OSX_VERSION >= 9 )); then
+  echo "Detected Mavericks (10.9)"
+elif (( $OSX_VERSION >= 8 )); then
+  echo "Detected Mountain Lion (10.8)"
 elif (( $OSX_VERSION >= 7 )); then
   echo "Detected Lion (10.7)"
 else
@@ -490,16 +883,17 @@ elif $USING_CLANG; then
   echo "Using clang compiler"
   export CC=clang
   export CXX=clang++
-  export QMAKESPEC=unsupported/macx-clang
 fi
 
 echo "Building for $MAC_OSX_VERSION_MIN or later"
 
+if [ ! $NUMCPU ]; then
+  NUMCPU=$(sysctl -n hw.ncpu)
+  echo "Setting number of CPUs to $NUMCPU"
+fi
+
 if $OPTION_DEPLOY; then
   echo "Building deployment version of libraries"
-  OPTION_32BIT=true
-else
-  OPTION_32BIT=false
 fi
 
 if $OPTION_32BIT; then
@@ -508,22 +902,32 @@ else
   echo "Building 64-bit binaries"
 fi
 
+if (( $OPTION_FORCE )); then
+  echo "Forcing rebuild"
+fi
+
 echo "Using basedir:" $BASEDIR
 mkdir -p $SRCDIR $DEPLOYDIR
-build_qt 4.8.5
-# NB! For eigen, also update the path in the function
-build_eigen 3.2.0
-build_gmp 5.1.3
-build_mpfr 3.1.2
-build_boost 1.54.0
-# NB! For CGAL, also update the actual download URL in the function
-build_cgal 4.3
-build_glew 1.10.0
-build_gettext 0.18.3.1
-build_libffi 3.0.13
-build_glib2 2.38.2
-build_opencsg 1.3.2
+
+# Only build deploy packages in deploy mode
 if $OPTION_DEPLOY; then
-#  build_sparkle andymatuschak 0ed83cf9f2eeb425d4fdd141c01a29d843970c20
-  build_sparkle Cocoanetics 1e7dcb1a48b96d1a8c62100b5864bd50211cbae1
+  # Array concatenation
+  PACKAGES=("${PACKAGES[@]}" "${DEPLOY_PACKAGES[@]}")
 fi
+
+# Build specified (or all) packages
+ALL_PACKAGES=$(all_packages)
+if [ -z "$OPTION_PACKAGES" ]; then
+  OPTION_PACKAGES=$ALL_PACKAGES
+fi
+
+echo "Building packages: $OPTION_PACKAGES"
+echo
+
+for package in $OPTION_PACKAGES; do
+  if [[ $ALL_PACKAGES =~ $package ]]; then
+    build $package $(package_version $package)
+  else
+    echo "Skipping unknown package $package"
+  fi
+done
