@@ -1,7 +1,10 @@
 #!/bin/sh
-
-# NB! To build a release build, the VERSION and VERSIONDATE environment variables needs to be set.
-# See doc/release-checklist.txt
+#
+# Usage:
+#   ./scripts/publish-macosx.sh [buildonly]
+#
+VERSION=2015.03
+VERSIONDATE=2015.03.06
 
 export NUMCPU=$(sysctl -n hw.ncpu)
 
@@ -24,12 +27,12 @@ update_www_download_links()
     webdir=../openscad.github.com
     # FIXME: release vs. snapshot
     incfile=inc/mac_snapshot_links.js
-    BASEURL='http://files.openscad.org/'
+    BASEURL='http://files.openscad.org/snapshots'
     DATECODE=`date +"%Y.%m.%d"`
     
     if [ -f $webdir/$incfile ]; then
         cd $webdir
-        echo "fileinfo['MAC_SNAPSHOT_URL'] = '$BASEURL$packagefile'" > $incfile
+        echo "fileinfo['MAC_SNAPSHOT_URL'] = '$BASEURL/$packagefile'" > $incfile
         echo "fileinfo['MAC_SNAPSHOT_NAME'] = 'OpenSCAD $version'" >> $incfile
         echo "fileinfo['MAC_SNAPSHOT_SIZE'] = '$filesize'" >> $incfile
         echo 'modified mac_snapshot_links.js'
@@ -41,6 +44,13 @@ update_www_download_links()
     fi
 }
 
+# Cmd-line parameters
+DOUPLOAD=1
+if [ "`echo $* | grep buildonly`" ]; then
+  echo "Build only, no upload"
+  DOUPLOAD=
+fi
+
 if test -z "$VERSIONDATE"; then
   VERSIONDATE=`date "+%Y.%m.%d"`
 fi
@@ -61,7 +71,7 @@ PATH=${PATH//\/opt\/local\/libexec\/ccache:}
 export MACOSX_DEPLOYMENT_TARGET=10.6
 
 # This is the same location as DEPLOYDIR in macosx-build-dependencies.sh
-export OPENSCAD_LIBRARIES=$PWD/../libraries/homebrew
+export OPENSCAD_LIBRARIES=$PWD/../libraries/install
 
 # Make sure that the correct Qt tools are used
 export PATH=$OPENSCAD_LIBRARIES/bin:$PATH
@@ -77,6 +87,10 @@ if [[ $? != 0 ]]; then
   exit 1
 fi
 
+if [ ! $DOUPLOAD ]; then
+  exit 0
+fi
+
 SIGNATURE=$(openssl dgst -sha1 -binary < OpenSCAD-$VERSION.dmg  | openssl dgst -dss1 -sign $HOME/.ssh/openscad-appcast.pem | openssl enc -base64)
 
 if [[ $VERSION == $VERSIONDATE ]]; then
@@ -93,10 +107,24 @@ if [[ $VERSION == $VERSIONDATE ]]; then
 fi
 
 echo "Uploading..."
-scp OpenSCAD-$VERSION.dmg openscad@files.openscad.org:www
+if [[ $VERSION == $VERSIONDATE ]]; then
+  scp OpenSCAD-$VERSION.dmg openscad@files.openscad.org:www/snapshots
+else
+  scp OpenSCAD-$VERSION.dmg openscad@files.openscad.org:www
+fi
 if [[ $? != 0 ]]; then
   exit 1
 fi
+scp $APPCASTFILE openscad@files.openscad.org:www/
+if [[ $? != 0 ]]; then
+  exit 1
+fi
+if [[ $VERSION == $VERSIONDATE ]]; then
+  scp $APPCASTFILE openscad@files.openscad.org:www/appcast-snapshots.xml
+  if [[ $? != 0 ]]; then
+    exit 1
+  fi
+fi
 
 # Update snapshot filename on web page
 update_www_download_links version=$VERSION packagefile=OpenSCAD-$VERSION.dmg filesize=$FILESIZE
