@@ -2,8 +2,7 @@
  * Copyright (C) 2008 Remko Troncon. BSD license
  * Copyright (C) 2013 Marius Kintel. BSD license
  */
-#ifndef SPARKLEAUTOUPDATER_H
-#define SPARKLEAUTOUPDATER_H
+#pragma once
 
 #include <QString>
 
@@ -11,6 +10,7 @@
 
 class SparkleAutoUpdater : public AutoUpdater
 {
+	Q_OBJECT;
 public:
 	SparkleAutoUpdater();
 	~SparkleAutoUpdater();
@@ -28,5 +28,3 @@ private:
 	class Private;
 	Private *d;
 };
-
-#endif
